==> flashtool-2.4.0-py3-none-any.whl: flashtool/products/kmb/kmbusb.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020, Intel Corporation. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause
#

from subprocess import run, PIPE
import time
import os
import logging
from flashtool.thirdparty.fastboot import Fastboot
from flashtool.products.kmb.kmberrors import NoDevicesFoundError, DeviceModeError
from flashtool.common.utils import async_dev_ops, find_any


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ToolError(Exception):
    pass


class DeviceDownloadError(ToolError):
    def __init__(self, err_str, dev):
        self.err_str = err_str
        self.dev = dev

    def __str__(self):
        return "Download failed for device:{} due to {}".format(self.dev, self.err_str)


class _KMBUSBdevice:

    gpt_cmd = ["./fastboot", "oem", "format"]
    flash_cmd = ["./fastboot", "flash", "partition", "image"]
    reboot_cmd = ["./fastboot", "reboot"]
    result_parse = r"^Finished. Total time: (\d+.\d+)s"
    status_dict = {"bl1": "Fip recovery", "Variable not implemented": "OS recovery", "bl33": "OS recovery"}

    def __init__(self, name, serialno):
        self.logger = logger
        self.name = name
        self.serialno = serialno
        self._status = None
        self._busnum = None
        self._devnum = None

    @property
    def status(self):
        if self._status is None:
            status_val = Fastboot(serial=self.serialno, name=self.name).getvar('Bootstage')
            self._status = self.status_dict[status_val]
        return self._status

    @property
    def sysfs_path(self):
        self._sysfs_path = find_any('*{}*'.format(self.name), '/sys/devices')
        return self._sysfs_path

    @property
    def bus_dev(self):
        if not self._busnum:
            with open(os.path.join(self.sysfs_path, 'busnum'), "r") as f:
                self._busnum = int(f.read().strip("\n"))
        if not self._devnum:
            with open(os.path.join(self.sysfs_path, 'devnum'), "r") as f:
                self._devnum = int(f.read().strip("\n"))
        return "{:03d}:{:03d}".format(self._busnum, self._devnum)

    def flash_fip(self, fip):
        if self.status != "Fip recovery":
            raise DeviceModeError("Fip recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).stage(fip)

    def flash_partition(self, part_name, img):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)

        # Retrieve its own image if there are multiple
        final_image = None
        if type(img) == list:
            if 'factory.img' == img[0]:
                final_image = img[0]
            else:
                # Find its own factory img
                for factory in img:
                    usb_serial = factory.split('factory_')[1].split('.img')[0]
                    if self.serialno == usb_serial:
                        final_image = factory
        else:
            final_image = img

        if final_image is None:
            raise RuntimeError(f'factory img for "{self.serialno}" device not specified')

        self.logger.debug(f'About to flash partition: {part_name}, file: {final_image} to device: {self.serialno}')

        # Flash to the exact device
        return Fastboot(serial=self.serialno, name=self.name).flash(part_name, final_image)

    def format(self):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).format()

    def erase(self, part_name):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).erase(part_name)

    def reboot(self):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).reboot()

    def __repr__(self):
        return "{} Intel Keembay USB(in {} mode): {}".format(self.name, self.status, self.serialno)


class KMBUSBManager:
    def __init__(self, slot=None):
        self.logger = logger
        self._devices = None
        self.slot = slot

    def _get_device_list(self):
        devices = [_KMBUSBdevice(device[0], device[1]) for device in Fastboot().devices()]
        self.logger.debug("USB device list: {}".format(devices))
        return devices

    @property
    def devices(self):
        if self._devices is None:
            self._devices = self._get_device_list()
        devices = [device for device in list(self._devices) if self.slot is None or device.name == self.slot]
        if devices:
            return devices
        else:
            raise NoDevicesFoundError(device_type='USB')

    def recover(self, fip):
        # Add check before and after. Same devices must be ready in OS mode
        # TODO(themarpe) - modify to actual serialno matching, instead of just num devices
        num_prev_devices = len(self.devices)

        self.logger.debug("Starting FIP flashing")
        for device, data in async_dev_ops(self.devices, 'flash_fip', fip).items():
            self.logger.info('{} completed FIP download in {:.4f}s'.format(device, data))
        self.logger.warning("Waiting 30s before reenumarating")
        time.sleep(30)
        self.rescan()
        if len(self.devices) != num_prev_devices:
            raise RuntimeError(f'Missing devices going from FIP -> OS recovery mode... Retry')
        for device in self.devices:
            if device.status == "OS recovery":
                self.logger.info("FIP flashed successfully for device {}".format(device))
            else:
                raise DeviceModeError("OS recovery", device)

    def format(self):
        self.logger.debug("Trigger GPT Creation")
        for device, data in async_dev_ops(self.devices, 'format').items():
            self.logger.info('{} created GPT in {:.4f}s'.format(device, data))

    def flash(self, part_name, img):
        self.logger.debug("Flashing {}".format(part_name))
        for device, data in async_dev_ops(self.devices, 'flash_partition', part_name, img).items():
            self.logger.info('{} flashed {} in {:.4f}s'.format(device, part_name, data))

    def erase(self, part_name):
        self.logger.debug("Erasing {}".format(part_name))
        for device, data in async_dev_ops(self.devices, 'erase', part_name).items():
            self.logger.info('{} erased {} in {:.4f}s'.format(device, part_name, data))

    def reboot(self):
        self.logger.debug("Signalling recovery done")
        for device, data in async_dev_ops(self.devices, 'reboot').items():
            self.logger.debug('{} signalled recovery done in {:.4f}s'.format(device, data))
            self.logger.info("OS flashing done for device {}. Please shutdown the system, set the boot switch to normal and reboot the system".format(device))

    def flash_all(self, **kwargs):
        fip = kwargs.get("fip", None)
        if fip:
            self.recover(fip)

        if not kwargs.get("skip_format", False):
            self.format()

        if kwargs.get("wipe", False):
            self.erase("env-main")
            self.erase("env-redund")

        partition_img_mapping = {
            "boot": "boot_a",
            "system": "system_a",
            "syshash": "syshash_a",
            "data": "data",
            "factory": "factory",
        }
        secondary_mapping = {
            "boot": "boot_b",
            "system": "system_b",
            "syshash": "syshash_b",
        }

        for key, val in kwargs.items():
            try:
                part_name = partition_img_mapping[key]
                image = val
            except KeyError:
                continue
            self.flash(part_name, image)
            if not kwargs.get("skip_secondary", False):
                try:
                    part_name = secondary_mapping[key]
                    self.flash(part_name, image)
                except KeyError:
                    pass

        if not kwargs.get("skip_boot", False):
            self.reboot()

    def rescan(self):
        for device in self._devices:
            del(device)
        self._devices = None

==> flashtool-2.5.0-py3-none-any.whl: flashtool/products/kmb/kmbusb.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020, Intel Corporation. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause
#

from subprocess import run, PIPE
import time
import os
import logging
from flashtool.thirdparty.fastboot import Fastboot
from flashtool.products.kmb.kmberrors import NoDevicesFoundError, DeviceModeError
from flashtool.common.utils import async_dev_ops, find_any


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ToolError(Exception):
    pass


class DeviceDownloadError(ToolError):
    def __init__(self, err_str, dev):
        self.err_str = err_str
        self.dev = dev

    def __str__(self):
        return "Download failed for device:{} due to {}".format(self.dev, self.err_str)


class _KMBUSBdevice:

    gpt_cmd = ["./fastboot", "oem", "format"]
    flash_cmd = ["./fastboot", "flash", "partition", "image"]
    reboot_cmd = ["./fastboot", "reboot"]
    result_parse = r"^Finished. Total time: (\d+.\d+)s"
    status_dict = {"bl1": "Fip recovery", "Variable not implemented": "OS recovery", "bl33": "OS recovery"}

    def __init__(self, name, serialno):
        self.logger = logger
        self.name = name
        self.serialno = serialno
        self._status = None
        self._busnum = None
        self._devnum = None

    @property
    def status(self):
        if self._status is None:
            status_val = Fastboot(serial=self.serialno, name=self.name).getvar('Bootstage')
            self._status = self.status_dict[status_val]
        return self._status

    @property
    def sysfs_path(self):
        self._sysfs_path = find_any('*{}*'.format(self.name), '/sys/devices')
        return self._sysfs_path

    @property
    def bus_dev(self):
        if not self._busnum:
            with open(os.path.join(self.sysfs_path, 'busnum'), "r") as f:
                self._busnum = int(f.read().strip("\n"))
        if not self._devnum:
            with open(os.path.join(self.sysfs_path, 'devnum'), "r") as f:
                self._devnum = int(f.read().strip("\n"))
        return "{:03d}:{:03d}".format(self._busnum, self._devnum)

    def flash_fip(self, fip):
        if self.status != "Fip recovery":
            raise DeviceModeError("Fip recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).stage(fip)

    def flash_partition(self, part_name, img):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)

        # Retrieve its own image if there are multiple
        final_image = None
        if type(img) == list:
            if 'factory.img' == img[0]:
                final_image = img[0]
            else:
                # Find its own factory img
                for factory in img:
                    usb_serial = factory.split('factory_')[1].split('.img')[0]
                    if self.serialno == usb_serial:
                        final_image = factory
        else:
            final_image = img

        if final_image is None:
            raise RuntimeError(f'factory img for "{self.serialno}" device not specified')

        self.logger.debug(f'About to flash partition: {part_name}, file: {final_image} to device: {self.serialno}')

        # Flash to the exact device
        return Fastboot(serial=self.serialno, name=self.name).flash(part_name, final_image)

    def format(self):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).format()

    def erase(self, part_name):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).erase(part_name)

    def reboot(self):
        if self.status != "OS recovery":
            raise DeviceModeError("OS recovery", self)
        return Fastboot(serial=self.serialno, name=self.name).reboot()

    def __repr__(self):
        return "{} Intel Keembay USB(in {} mode): {}".format(self.name, self.status, self.serialno)


class KMBUSBManager:
    def __init__(self, slot=None):
        self.logger = logger
        self._devices = None
        self.slot = slot

    def _get_device_list(self):
        devices = [_KMBUSBdevice(device[0], device[1]) for device in Fastboot().devices()]
        self.logger.debug("USB device list: {}".format(devices))
        return devices

    @property
    def devices(self):
        if self._devices is None:
            self._devices = self._get_device_list()
        devices = [device for device in list(self._devices) if self.slot is None or device.name == self.slot]
        if devices:
            return devices
        else:
            raise NoDevicesFoundError(device_type='USB')

    def recover(self, fip):
        # Add check before and after. Same devices must be ready in OS mode
        prev_devices = self.devices.copy()

        self.logger.debug("Starting FIP flashing")
        for device, data in async_dev_ops(self.devices, 'flash_fip', fip).items():
            self.logger.info('{} completed FIP download in {:.4f}s'.format(device, data))
        self.logger.warning("Waiting 30s before reenumarating")
        time.sleep(30)
        self.rescan()

        # check which devices are missing
        missing_devices = []
        for req_dev in prev_devices:
            found = False
            for new_dev in self.devices:
                if req_dev.serialno == new_dev.serialno:
                    found = True
                    break
            if not found:
                missing_devices.append(req_dev)

        if len(missing_devices) > 0:
            raise RuntimeError(f'Missing devices going from FIP -> OS recovery mode... Retry. Problematic devices: {missing_devices}')
        for device in self.devices:
            if device.status == "OS recovery":
                self.logger.info("FIP flashed successfully for device {}".format(device))
            else:
                raise DeviceModeError("OS recovery", device)

    def format(self):
        self.logger.debug("Trigger GPT Creation")
        for device, data in async_dev_ops(self.devices, 'format').items():
            self.logger.info('{} created GPT in {:.4f}s'.format(device, data))

    def flash(self, part_name, img):
        self.logger.debug("Flashing {}".format(part_name))
        for device, data in async_dev_ops(self.devices, 'flash_partition', part_name, img).items():
            self.logger.info('{} flashed {} in {:.4f}s'.format(device, part_name, data))

    def erase(self, part_name):
        self.logger.debug("Erasing {}".format(part_name))
        for device, data in async_dev_ops(self.devices, 'erase', part_name).items():
            self.logger.info('{} erased {} in {:.4f}s'.format(device, part_name, data))

    def reboot(self):
        self.logger.debug("Signalling recovery done")
        for device, data in async_dev_ops(self.devices, 'reboot').items():
            self.logger.debug('{} signalled recovery done in {:.4f}s'.format(device, data))
            self.logger.info("OS flashing done for device {}. Please shutdown the system, set the boot switch to normal and reboot the system".format(device))

    def flash_all(self, **kwargs):
        fip = kwargs.get("fip", None)
        if fip:
            self.recover(fip)

        if not kwargs.get("skip_format", False):
            self.format()

        if kwargs.get("wipe", False):
            self.erase("env-main")
            self.erase("env-redund")

        partition_img_mapping = {
            "boot": "boot_a",
            "system": "system_a",
            "syshash": "syshash_a",
            "data": "data",
            "factory": "factory",
        }
        secondary_mapping = {
            "boot": "boot_b",
            "system": "system_b",
            "syshash": "syshash_b",
        }

        for key, val in kwargs.items():
            try:
                part_name = partition_img_mapping[key]
                image = val
            except KeyError:
                continue
            self.flash(part_name, image)
            if not kwargs.get("skip_secondary", False):
                try:
                    part_name = secondary_mapping[key]
                    self.flash(part_name, image)
                except KeyError:
                    pass

        if not kwargs.get("skip_boot", False):
            self.reboot()

    def rescan(self):
        for device in self._devices:
            del(device)
        self._devices = None
